@@ -1,13 +1,17 @@
 # TODO: determine if provisioning should be implicit like it is now, or if it should be explicitly done
+import json
+import os
 import random
 import string
 
+from pathlib import Path
+
 from python_soql_parser import parse
 
-from simple_salesforce.exceptions import SalesforceResourceNotFound
-
-from simple_mockforce.query import (
+from simple_mockforce.query_algorithms import (
+    add_parent_object_attributes,
     filter_by_where_clause,
+    sort_by_order_by_clause,
 )
 from simple_mockforce.utils import find_object_and_index
 
@@ -25,9 +29,18 @@ class VirtualSalesforce:
     """
 
     def __init__(self):
-        self.create_new_virtual_instance()
+        self.provision()
 
-    def create_new_virtual_instance(self):
+        # temporary support for related object field names that don't
+        # match the name of the related object, e.g., we can do Account__r,
+        # but need this for when Company__r corresponds to an Account object
+        path_to_relations_root = os.getenv("MOCKFORCE_RELATIONS_ROOT", ".")
+        path_to_relations_json = Path(path_to_relations_root) / "relations.json"
+        with open(path_to_relations_json) as file:
+            relations_file = json.load(file)
+        self.relations_file = relations_file
+
+    def provision(self):
         """
         Starts a virtual Salesforce instance from scratch. Useful to prevent test pollution
         """
@@ -55,20 +68,41 @@ class VirtualSalesforce:
             sobject
         ), f"{parsed_sobject} not present in the virtual Salesforce objects"
 
-        fields = parse_results["fields"].asList()
+        all_fields = parse_results["fields"].asList()
+        fields = list()
+        parent_fields = list()
+
+        for field in all_fields:
+            if "." in field:
+                parent_fields.append(field)
+            else:
+                fields.append(field)
+
         where = parse_results["where"].asList()
 
         limit = parse_results["limit"].asList()
+
+        # TODO: why do we need to check?
+        order_by = (
+            parse_results["order_by"].asList() if "order_by" in parse_results else None
+        )
         sobjects = self.get_sobjects(sobject)
 
         records = list()
 
+        if order_by:
+            sort_by_order_by_clause(sobjects, order_by)
+
         for sobject in sobjects:
             passes = filter_by_where_clause(sobject, where)
             if not passes:
                 continue
 
             record = {field: sobject.get(field) for field in fields}
+
+            if parent_fields:
+                add_parent_object_attributes(sobject, record, parent_fields, self)
+
             records.append(record)
 
         if limit:
@@ -177,23 +211,37 @@ class VirtualSalesforce:
             if key.endswith("__r"):
                 # We assume there's only one key in this dict
                 for external_id_field, external_id in value.items():
-                    related_object_name = key.replace("__r", "__c")
+                    relational_field_name = self._related_object_name_to_object_name(
+                        key
+                    )
+                    # if the related object name can't be inferred from the field name
+                    # check the manually specified mapping
+                    if key in self.relations_file:
+                        related_object_name = self.relations_file[key]
+                    # o/w infer it automatically
+                    else:
+                        related_object_name = relational_field_name
                     # If this is a standard object, we have to pop-off the __c
-                    # A little dirty for sure
+                    # A little dirty for sure, especially since we overwrite relational_field_name
                     standard_object_name = related_object_name.replace("__c", "")
                     if (
                         related_object_name not in self.data
                         and standard_object_name in self.data
                     ):
                         related_object_name = standard_object_name
+                        relational_field_name = standard_object_name
                     related_object = self.get_by_custom_id(
                         related_object_name, external_id, external_id_field
                     )
-                    normalized[related_object_name] = related_object["Id"]
+                    normalized[relational_field_name] = related_object["Id"]
             else:
                 normalized[key] = value
         return normalized
 
+    @staticmethod
+    def _related_object_name_to_object_name(related_object_name: str):
+        return related_object_name.replace("__r", "__c")
+
     def _provision_sobject(self, sobject_name: str):
         """
         Provisions a virtual Salesfoce object
@@ -208,4 +256,4 @@ class VirtualSalesforce:
         return "".join(random.choices(string.ascii_letters + string.digits, k=18))
 
 
-virtual_salesforce = VirtualSalesforce()
\ No newline at end of file
+virtual_salesforce = VirtualSalesforce()
